use hyper::{Body as RawBody, Chunk};
use futures::Poll;
use futures::stream::Stream;
use errors::HandleError;
use super::from_body::FromBody;


pub struct Body {
    raw: RawBody,
}

impl Body {
    pub fn into_value<T: FromBody>(self) -> T::Future {
        T::from_body(self)
    }
}

impl From<RawBody> for Body {
    fn from(raw: RawBody) -> Self {
        Body { raw }
    }
}

impl Stream for Body {
    type Item = Chunk;
    type Error = HandleError;

    fn poll(&mut self) -> Poll<Option<Self::Item>, Self::Error> {
        self.raw.poll().map_err(HandleError::internal)
    }
}
