use futures::{future, Future, Stream};
use futures::future::BoxFuture;
use hyper::mime::{self, Mime};
use errors::HandleError;
use super::fold_fn;
use super::body::Body;

pub trait FromBody: Sized {
    type Future: Send + 'static + Future<Item = Self, Error = HandleError>;
    fn check_mime(m: &Mime) -> bool;
    fn from_body(body: Body) -> Self::Future;
}


impl FromBody for Vec<u8> {
    type Future = BoxFuture<Vec<u8>, HandleError>;

    fn check_mime(m: &Mime) -> bool {
        *m == mime::APPLICATION_OCTET_STREAM
    }

    fn from_body(body: Body) -> Self::Future {
        body.fold(Vec::new(), fold_fn).boxed()
    }
}

impl FromBody for String {
    type Future = BoxFuture<String, HandleError>;

    fn check_mime(m: &Mime) -> bool {
        *m == mime::TEXT_PLAIN_UTF_8
    }

    fn from_body(body: Body) -> Self::Future {
        body.fold(Vec::new(), fold_fn)
            .and_then(|body| {
                future::result(String::from_utf8(body).map_err(|_| {
                    HandleError::bad_request()
                }))
            })
            .boxed()
    }
}
