use context::Context;
use outcome::Outcome;
use response::Response;

/// Represents a handler
pub trait Handler: 'static + Send + Sync {
    fn call(&self, ctx: Context) -> Outcome<Response>;
}

impl<F> Handler for F
where
    F: 'static + Send + Sync + Fn(Context) -> Outcome<Response>,
{
    fn call(&self, ctx: Context) -> Outcome<Response> {
        (*self)(ctx)
    }
}


/// Represents a middleware.
pub trait Middleware: 'static + Send + Sync {
    fn call(&self, ctx: Context) -> Outcome<Context>;
}
