//!
//! A micro Web framework based on Hyper, Futures and Tokio.
//!
//! # Example
//! ```{rust,ignore}
//! extern crate susanoo;
//!
//! use susanoo::prelude::*;
//!
//! fn hello(ctx: Context) -> Outcome<Response> {
//!     ctx.respond("Hello, world")
//! }
//!
//! fn main() {
//!     let s = Susanoo::default()
//!         .with_route(Route::get("/", hello));
//!     s.run().unwrap()
//! }
//! ```

pub extern crate futures;
pub extern crate hyper;
#[doc(hidden)]
pub extern crate tokio_core;
pub extern crate typemap;
#[doc(hidden)]
pub extern crate url;
#[macro_use]
extern crate error_chain;
extern crate regex;
extern crate tokio_proto;

pub mod body;
pub mod context;
pub mod errors;
pub mod handler;
pub mod outcome;
pub mod request;
pub mod response;
pub mod router;
pub mod susanoo;

// re-exports

#[doc(inline)]
pub use hyper::header;
#[doc(inline)]
pub use hyper::StatusCode;

#[doc(inline)]
pub use body::{Body, FromBody, FromForm};
#[doc(inline)]
pub use context::Context;
#[doc(inline)]
pub use errors::HandleError;
#[doc(inline)]
pub use handler::{Handler, Middleware};
#[doc(inline)]
pub use outcome::Outcome;
#[doc(inline)]
pub use request::Request;
#[doc(inline)]
pub use response::{Response, Responder};
#[doc(inline)]
pub use router::Route;
#[doc(inline)]
pub use susanoo::Susanoo;

pub mod prelude {
    pub use futures::{Future, Stream};
    pub use context::Context;
    pub use handler::{Handler, Middleware};
    pub use outcome::Outcome;
    pub use response::Response;
    pub use router::Route;
    pub use susanoo::Susanoo;
    pub use errors::HandleError;
}
