use hyper::header::Location;
use hyper::{Response as RawResponse, StatusCode};
use request::Request;
use errors::HandleError;
use super::responder::Responder;
use super::response::Response;


/// A responder represents a redirect to a given URL.
pub struct Redirect {
    status: StatusCode,
    uri: String,
}

impl Redirect {
    /// Construct "301 Moved Permanently"
    pub fn moved<S: Into<String>>(uri: S) -> Self {
        Redirect {
            status: StatusCode::MovedPermanently,
            uri: uri.into(),
        }
    }

    /// Construct "302 Found"
    pub fn found<S: Into<String>>(uri: S) -> Self {
        Redirect {
            status: StatusCode::Found,
            uri: uri.into(),
        }
    }

    /// Construct "303 See Other"
    pub fn see_other<S: Into<String>>(uri: S) -> Self {
        Redirect {
            status: StatusCode::SeeOther,
            uri: uri.into(),
        }
    }

    /// Construct "305 Use Proxy"
    pub fn use_proxy<S: Into<String>>(uri: S) -> Self {
        Redirect {
            status: StatusCode::UseProxy,
            uri: uri.into(),
        }
    }

    /// Construct "307 Temporary Redirect"
    pub fn temporary<S: Into<String>>(uri: S) -> Self {
        Redirect {
            status: StatusCode::TemporaryRedirect,
            uri: uri.into(),
        }
    }

    /// Construct "308 Permanent"
    pub fn permanent<S: Into<String>>(uri: S) -> Self {
        Redirect {
            status: StatusCode::PermanentRedirect,
            uri: uri.into(),
        }
    }
}

impl Responder for Redirect {
    fn respond(self, _: &Request) -> Result<Response, HandleError> {
        Ok(
            RawResponse::new()
                .with_status(self.status)
                .with_header(Location::new(self.uri))
                .into(),
        )
    }
}
