use hyper::{Response as RawResponse, StatusCode};
use hyper::header::ContentType;
use request::Request;
use errors::HandleError;
use super::response::Response;


/// Represents a "responder", which converted to `hyper::Response`.
pub trait Responder {
    /// Convert itself to `hyper::Response`.
    fn respond(self, req: &Request) -> Result<Response, HandleError>;
}

impl Responder for RawResponse {
    #[inline(always)]
    fn respond(self, _: &Request) -> Result<Response, HandleError> {
        Ok(self.into())
    }
}

impl Responder for &'static str {
    fn respond(self, _: &Request) -> Result<Response, HandleError> {
        Ok(
            RawResponse::new()
                .with_header(ContentType::plaintext())
                .with_body(self)
                .into(),
        )
    }
}

impl Responder for String {
    fn respond(self, _: &Request) -> Result<Response, HandleError> {
        Ok(
            RawResponse::new()
                .with_header(ContentType::plaintext())
                .with_body(self)
                .into(),
        )
    }
}

impl Responder for () {
    fn respond(self, _: &Request) -> Result<Response, HandleError> {
        Ok(
            RawResponse::new()
                .with_status(StatusCode::NoContent)
                .into(),
        )
    }
}
